/**
 * Bluetooth.java
 */

import java.io.IOException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;

/**
 * Copyright (c) 2003 Sony Ericsson Mobile Communications AB<br>
 * All Rights Reserved.
 * <p>
 * BluetoothCar is the main program. It creates a SPPServer, a GUI and a CarAPI.
 * The CarAPI is a parameter when creating SPPServer and GUI. 
 *
 * @author	Christopher Wong
 * @author	ke Rydgren
 * @version	1.14
 */
public class BluetoothCar extends MIDlet {
	// String that is displayed in the about box:
	public static final String aboutString = 
		"Copyright (c) 2003\n" +
		"Sony Ericsson Mobile Communications AB\n" +
		"All Rights Reserved.\n\n" + 
		"This application is free of charge and provided as is, without any kind of warranties.\n\n" +
		"The purpose of this application is to demonstrate the possibilities with Java MIDP 2.0 " +
		"and the Sony Ericsson Bluetooth car, CAR-100.\n\n" +
		"Written in 2003 by Christopher Wong and ke Rydgren";
    
	// String that contains the MIDlet's version (displayed in the about box):
	//**************************************************************************************
	//******************* REMEMBER TO CHANGE THIS TO THE CURRENT VERSION *******************
	//**************************************************************************************
	public static final String versionString = "1.15";

	// Used by the MIDlet system
	private static BluetoothCar instance = null;
	
    // The display for this MIDlet
    private Display display;	
    
    // The bridge between GUI and SPPServer
    private CarAPI carAPI;
    
    // This MIDlet's canvas.	
    private GUI gui;		
    
    // Bluetooth Serial Port Profile server.
	private SPPServer spp;
	
	/** 
     * Constructs a new <code>BluetoothCar</code> object.
     */
    public BluetoothCar() {
    	// The instance used by MIDlet
    	instance = this;
    } // End of Constructor.

	/**
	 * Checks if the GUI has processed all pending car events.
	 *
	 * @return true if the GUI has processed all pending car events.
	 */
	public boolean guiUpdated() {
		if (gui != null)
			return gui.allUpdated();
		else
			return false;
	}
	
  	/**
  	 * This method is used since the MIDP API requires that a 
  	 * <code>MIDlet</code> would set the screen.
  	 *
  	 * @return	this object.
  	 */
  	static BluetoothCar getInstance() {
		return(instance);
  	}
  
    /**
     * Signals the <code>MIDlet</code> that it has entered the 
     * <i>Active</i> state.
     */
    public void startApp() {
    	// Get current display (more safe to do it here than in the constructor)
    	if (display == null) { // First time...
    		display = Display.getDisplay(this);
	    	
	    	// Create the API used by GUI and SPPServer. The API acts as a 
	    	// communication bridge.
	    	carAPI = new CarAPI(this);

	    	// Create the user-interface.
	    	gui = new GUI(carAPI, this);
	    	
	    	// Create the Serial Port Profile server. It will start automatically.
			spp = new SPPServer(carAPI, this);
		}
    	// Display the GUI.
        display.setCurrent(gui);
        // Tell CarAPI that the MIDlet is active now.
		carAPI.setAppActive(true);
    }

    /**
     * Signals the <code>MIDlet</code> to enter <i>Paused</i> state.
     */
    public void pauseApp() {
		carAPI.setAppActive(false);
    }

    /**
     * This will cleanup and release all resources.
     *
     * @param unconditional whether it is <code>true</code> or 
     *						<code>false</code> the <code>MIDlet</code> will 
     *						cleanup	and exit.
     */
    public void destroyApp(boolean unconditional) {
    	carAPI.stopTimer();
    	carAPI.stopWaiting();
    	// Destroy the API used by GUI and SPPServer.
    	spp.closeAll();
  		spp = null;
   		gui = null;
    	carAPI = null;
    	display = null;
    	instance = null;
    	notifyDestroyed();
    }

}
